#include "stdlib.h"
#include "oric.h"
#include "chess.h"

unsigned char mycolor, othercolor=COLOR;
char kingmoved, tower0moved, tower7moved, en_passant;
char lastfromx, lastfromy, lasttox, lasttoy;
unsigned char board[8][8]={
     2,    3,    4,    5,    6,    4,    3,    2,
     1,    1,    1,    1,    1,    1,    1,    1,
     0,    0,    0,    0,    0,    0,    0,    0,
     0,    0,    0,    0,    0,    0,    0,    0,
     0,    0,    0,    0,    0,    0,    0,    0,
     0,    0,    0,    0,    0,    0,    0,    0,
  0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81, 0x81,
  0x82, 0x83, 0x84, 0x85, 0x86, 0x84, 0x83, 0x82  };

int no_between(int x, int y, int i, int j) {
  int k, l, sum=0, dx=SGN(x-i), dy=SGN(y-j);
  for (k=i+dx, l=j+dy ; k!=x || l!=y ; k+=dx, l+=dy) sum|=board[l][k];
  return (!sum);
}

int under_fire(int x, int y) {
  int i,j;
  for (i=0;i<8;i++) for (j=0;j<8;j++) {
    char dx=ABS(i-x), dy=ABS(j-y);
    if (board[j][i] && (board[j][i]&COLOR)!=mycolor)
      switch (board[j][i]&PIECE) {
	case KING:
	  if (dx<=1 && dy<=1) return 1;
	  break;
	case PAWN:
	  if (y==j-1 && dx==1) return 1;
	  break;
	case TOWER:
	  if ((dx==0 || dy==0) && no_between(x,y,i,j)) return 1;
	  break;
	case KNIGHT:
	  if (dx==2 && dy==1 || dx==1 && dy==2) return 1;
	  break;
	case BISHOP:
	  if (dx==dy && no_between(x,y,i,j)) return 1;
	  break;
	case QUEEN:
	  if ((dx==0 || dy==0 || dx==dy) && no_between(x,y,i,j))
	    return 1;
	  break;
      }
  }
  return 0;
}

int valid(int fromx, int fromy, int tox, int toy) {
  char dx=ABS(fromx-tox), dy=ABS(fromy-toy);
  char from=board[fromy][fromx], to=board[toy][tox];
  if (!from || (from&COLOR)!=mycolor || dx==0 && dy==0 
	|| to && (to&COLOR)==mycolor) return 0;
  switch (from&PIECE) {
    case KING:
      if (dx<=1 && dy<=1) return 1;
      if (!kingmoved && toy==0 && !under_fire(fromx,0)
		&& (tox==fromx-2 && !tower0moved && no_between(0,0,fromx,0)
			&& !under_fire(fromx-1,0)
		    || tox==fromx+2 && !tower7moved && no_between(fromx,0,7,0)
			&& !under_fire(fromx+1,0))) return 1;
      break;
    case PAWN:
      if (dx==0 && to==0 && (toy==fromy+1 || toy==3 && board[2][tox]==0)) 
	return 1;
      if (dx==1 && toy==fromy+1 && to) return 1;
      if (dx==1 && toy==fromy+1 && toy==5
		&& tox==lasttox && lasttoy==4 && (board[4][tox]&PIECE)==PAWN) {
	en_passant=1; return 1;
      }
      break;
    case TOWER:
      if ((dx==0 || dy==0) && no_between(tox,toy,fromx,fromy))
	return 1;
      break;
    case KNIGHT:
      if (dx==2 && dy==1 || dx==1 && dy==2) return 1;
      break;
    case BISHOP:
      if (dx==dy && no_between(tox,toy,fromx,fromy))
	return 1;
      break;
    case QUEEN:
      if ((dx==0 || dy==0 || dx==dy) && no_between(tox,toy,fromx,fromy)) 
	return 1;
      break;
  }
  return 0;
}

char promotion(int x) {
  int promotion=5;
  char c;
  while ((c=getchar())!=0x0D) {
    switch (c) {
      case 0x08: case 0x0B: promotion--; break;
      case 0x09: case 0x0A: promotion++; break;
      default:
	if (c>=32) { chat(c); send_char(c); }
    }
    if (promotion==1) promotion=5;
    if (promotion==6) promotion=2;
    board[7][x]=promotion|mycolor;
    display(x,7);
    c=receive_char(); if (c!=-1) display_chat(c);
  }
  return "0PTNBQK"[promotion];
}

void swap_sides(void) {
  int i,j;
  char tmp;
  for (j=0;j<4;j++) for (i=0;i<8;i++) {
    tmp=board[j][i]; board[j][i]=board[7-j][7-i]; board[7-j][7-i]=tmp;
  }
  tmp=mycolor; mycolor=othercolor; othercolor=tmp;
}

void move(int fromx, int fromy, int tox, int toy) {
  char to=board[toy][tox]=board[fromy][fromx];
  board[fromy][fromx]=0;
  display(fromx,fromy); display(tox,toy);
  switch (to&PIECE) {
    case KING:
      kingmoved=1;
      if (tox-fromx==2) {
	board[0][tox-1]=board[0][7]; board[0][7]=0;
	display(7,0); display(tox-1,0);
      }
      if (fromx-tox==2) {
	board[0][tox+1]=board[0][0]; board[0][0]=0;
	display(0,0); display(tox+1,0);
      }
      break;
    case TOWER:
      if (fromx==0 && fromy==0) tower0moved=1;
      if (fromx==7 && fromy==0) tower7moved=1;
      break;
    case PAWN:
      if (en_passant) {
	board[4][tox]=0; display(tox,4);
      }
      break;
  }
}

void send_move(int fromx,int fromy,int tox,int toy) {
  send_char(0x1B);
  if (mycolor) { fromx=7-fromx; fromy=7-fromy; tox=7-tox; toy=7-toy; }
  send_char('A'+fromy); send_char('1'+fromx);
  send_char('A'+toy); send_char('1'+tox);
  if (toy==7 && (board[toy][tox]&PIECE)==PAWN) send_char(promotion(tox));
  if (en_passant) { send_char('P'); en_passant=0; }
  send_char(';');
}

int opponent_play(void) {
  int fromx, fromy, tox, toy, special;
  special=wait_char(); 
  if (special=='+') {
    char min=peek(0xBF64);
    if (min!='0') poke(0xBF64,min=='9'?'0':min+1);
    return 0;
  } else if (special=='-') {
    char min=peek(0xBC1C);
    if (min!='1') poke(0xBC1C,min=='0'?'9':min-1);
    return 0;
  } else if (special=='S') {
    swap_sides();
    display_board();
    return 0;
  } else if (special<'A' || special>'H') return 0;
  fromy=special-'A'; fromx=wait_char()-'1';
  toy=wait_char()-'A'; tox=wait_char()-'1';
  if (mycolor) { fromx=7-fromx; fromy=7-fromy; tox=7-tox; toy=7-toy; }
  board[toy][tox]=board[fromy][fromx];
  board[fromy][fromx]=0; display(fromx,fromy);
  special=wait_char();
  if (special!=';') wait_char();
  switch (special) {
    case 'P': board[3][tox]=0; display(tox,3); break;
    case 'T': board[toy][tox]=TOWER|othercolor; break;
    case 'N': board[toy][tox]=KNIGHT|othercolor; break;
    case 'B': board[toy][tox]=BISHOP|othercolor; break;
    case 'Q': board[toy][tox]=QUEEN|othercolor; break;
  }
  display(tox,toy);
  if ((board[toy][tox]&PIECE)==KING && ABS(tox-fromx)==2) {
    if (tox<4) {
      board[7][0]=0; board[7][tox+1]=TOWER|othercolor;
      display(0,7); display(tox+1,7);
    } else {
      board[7][7]=0; board[7][tox-1]=TOWER|othercolor;
      display(7,7); display(tox-1,7);
    }
  }
  lasttox=tox; lasttoy=toy; lastfromx=fromx; lastfromy=toy;
  return 1;
}
